 #version 330 core
layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;
in vec2 TexCoords;
in vec3 TangentNormal;
in vec3 FragPos;
in vec3 TangentLightPos;
in vec3 TangentViewPos;
in vec3 TangentFragPos;
in vec4 PostPerspectiveFragPos;

uniform samplerCube depthMap;
uniform sampler2D texture_diffuse;
uniform sampler2D texture_normal;

uniform bool useNormalMapping;

uniform vec3 lightColor;
uniform vec3 lightPos; 
uniform vec4 postPerspectiveLightPos;
uniform vec3 viewPos;

uniform float farPlane;


// array of offset direction for sampling for PCF algorithm
vec3 sampleOffsetDirections[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

//returns 1 if fragment is shadow, 0 when it is not in shadow
float PointShadowCalculation(){

	//retrieve depth of the cubemap. Direction is the direction from fragPos to lightPos
	vec4 fragToLight = PostPerspectiveFragPos - postPerspectiveLightPos ;
	
	//Get normalized depth value between light source and closest visible fragment
	float closestDepth = texture(depthMap, fragToLight.xyz).r;
	//closestDepth is in range [0,1] - transform it back to [0,farPlane]
	closestDepth = closestDepth * farPlane;
	
	float currentDepth = length(fragToLight);
	if(currentDepth > closestDepth){
		return 1;
	}else{
		return 0;
	}
	/*
	//compare currentDepth and closestDepth
	//Apply PCF algorithm so that the shadow appears smoother
	float shadow = 0.0;
	float bias = 0.15;
	int samples = 20;	//sample in 20 different directions
	float viewDistance = length(viewPos - fragPos);
	//make radius dependent on the distance to the viewer
	float radius = (1.0 + (viewDistance / farPlane)) / 50.0 ;
	for(int i = 0; i < samples; ++i)
	{
		float closestDepth = texture(depthMap, fragToLight + sampleOffsetDirections[i]* radius).r;
		closestDepth *= farPlane; //Undo Mapping [0;1]
		if(currentDepth - bias > closestDepth){
			shadow += 1.0;
		}
	}
	shadow = shadow / float(samples);
	return shadow;
	*/
}


void main()
{   
	vec3 normal = normalize(TangentNormal);
	if(useNormalMapping){
		//Get normal out of the normal map in range [0.1]
		normal = texture(texture_normal, TexCoords).rgb;
		// Transform normal vector to range [-1,1]
		normal = normalize(normal * 2.0 - 1.0);	//This normal is in tangent space
	}
	//Perform phong shading
    vec4 objectColor = texture(texture_diffuse, TexCoords);
	//Add ambient lighting
    float ambientStrength = 0.3;
    vec3 ambient = ambientStrength * lightColor;

	//add diffuse lighting
	vec3 lightDir = normalize(TangentLightPos - TangentFragPos); 
	float diff = max(dot(normal, lightDir), 0.0);
	vec3 diffuse = diff * lightColor;

	//add specular lighting
    float specularStrength = 0.5;
	float shininess = 64;
	vec3 viewDir = normalize(TangentViewPos - TangentFragPos);
	vec3 reflectDir = reflect(-lightDir, normal);  
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
	vec3 specular = specularStrength * spec * lightColor;  
	
	//Calculate shadow
	float shadow = PointShadowCalculation();
	vec4 result = vec4((ambient + (1.0 - shadow) * (diffuse + specular)),1.0) * objectColor;
	FragColor = result;
	//FragColor = vec4(vec3(shadow/ farPlane), 1.0); Debug information
	 BrightColor = vec4(0.0,0.0,0.0,1.0);
}